
import { ImageMacro, ImageTemplate } from '../../storage/interface';
import { store } from '../../storage/store';
import { IncomingMessage, ServerResponse } from 'http';

export async function handle_image_request(req: IncomingMessage, res: ServerResponse) {
	if (req.url === '/' && req.method === 'GET') {
		res.writeHead(200, {
			'content-type': 'text/html',
		});
		res.end(`<!doctype html>
		<html>
		<head></head>
		<body>
			<h2>&lt;embed&gt;</h2>
			<embed type="image/svg+xml" src="http://me.local.jbrumond.me:54320/FrztglsLexLRWg"></embed>
		
			<h2>&lt;object&gt;</h2>
			<object type="image/svg+xml" data="http://me.local.jbrumond.me:54320/FrztglsLexLRWg"></object>
			
			<h2>&lt;iframe&gt;</h2>
			<iframe src="http://me.local.jbrumond.me:54320/FrztglsLexLRWg"></iframe>
		</body>
		</html>
		`);
		return;
	}

	if (! req.url.startsWith('/')) {
		return send_404_not_found(res);
	}

	switch (req.method) {
		case 'OPTIONS': return send_options_response(res);
		case 'GET': return send_image_response(res, await get_image(req));
		case 'HEAD': return send_image_response(res, await get_image(req), false);
	}

	return send_415_method_not_allowed(res);
}

function send_options_response(res: ServerResponse) {
	res.writeHead(200, {
		'access-control-allow-origin': '*',
		'access-control-allow-methods': 'GET, HEAD, OPTIONS',
	});
	res.end();
}

function send_image_response(res: ServerResponse, image: ImageTemplate | ImageMacro, send_content = true) {
	if (! image) {
		return send_404_not_found(res);
	}

	const buf = image.media_type === 'image/svg+xml'
		? Buffer.from(image.content, 'utf8')
		: image.content;

	res.writeHead(200, {
		'content-type': image.media_type,
		'content-length': buf.byteLength,
		'cache-control': 'public, max-age=31536000',
	});

	res.end(send_content ? buf : void 0);
}

function send_404_not_found(res: ServerResponse) {
	res.writeHead(404, {
		'content-type': 'text/plain'
	});
	res.end('Image not found');
}

function send_415_method_not_allowed(res: ServerResponse) {
	res.writeHead(415, {
		'content-type': 'text/plain'
	});
	res.end('Method not allowed');
}

function get_image(req: IncomingMessage) {
	const image_id = req.url.slice(1);
	return store.get_image_by_id(image_id);
}
